using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;


namespace test_capseditor
{
    public partial class Form1 : Form
    {
        #region constantes
        private const string standardTitle = "CapsEditor";
        private const uint margin = 10; // marge en hauteur avant d'afficher le dbut du texte
        #endregion 


        #region champs
        private List<TextLineInformation> documentLines = new List<TextLineInformation>();
        private uint lineHeight;
        private Size documentSize;
        private uint nLines;

        private Font mainFont;
        private Font emptyFont;
        private Brush mainBrush = Brushes.Blue;
        private Brush emptyBrush = Brushes.PowderBlue;

        private Point doubleClickPosition;
        private OpenFileDialog fileOpenDialog = new OpenFileDialog();

        private bool documentHasData = false;
        
        #endregion

        public Form1()
        {
            InitializeComponent();
            CreateFonts();
            this.BackColor = Color.White; 
            fileOpenDialog.FileOk += new CancelEventHandler(fileOpenDialog_FileOk);
            fileOpenDialog.Filter =
                "Fichiers Textes (*.txt, *.html)|*.txt;*.html;*.htm|" +
                "Codes sources (*.cs)|*.cs|"+
                "Tous les fichiers|*.*";
        }

        void fileOpenDialog_FileOk(object sender, CancelEventArgs e)
        {
            this.LoadFile(fileOpenDialog.FileName);
        }

        private void LoadFile(string fileName)
        {
            StreamReader sr = new StreamReader(fileName);
            string prochaineLigne;
            documentLines.Clear();
            nLines = 0;
            TextLineInformation infosProchaineLigne;

            while ((prochaineLigne = sr.ReadLine()) != null)
            {
                infosProchaineLigne = new TextLineInformation();
                infosProchaineLigne.Text = prochaineLigne;
                nLines++;
                documentLines.Add(infosProchaineLigne);
            }
            sr.Close();
            documentHasData = (nLines > 0) ? true : false;

            CalculateLineWidth();
            CalculateDocumentSize();

            this.Text = standardTitle + " - " + fileName;
            this.Invalidate();
        }

        private void CalculateDocumentSize()
        {
            if (!documentHasData)
            {
                documentSize = new Size(100, 200);
            }
            else
            {
                documentSize.Height = (int)(nLines * lineHeight) + 2 * (int)margin;
                uint maxLineWidth = 0;
                foreach (TextLineInformation nextLine in documentLines)
                {
                    if ((nextLine.Width + 2 * margin) > maxLineWidth)
                    { maxLineWidth = nextLine.Width+ 2 * margin; }
                }
                documentSize.Width = (int) maxLineWidth;
            }
            this.AutoScrollMinSize = documentSize;
        }

        private void CalculateLineWidth()
        {
            Graphics graph = this.CreateGraphics();
            foreach (TextLineInformation nextLine in documentLines)
            {
                nextLine.Width = (uint)graph.MeasureString(nextLine.Text, mainFont).Width;
            }

        }

        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);
            Graphics graph = e.Graphics;
            int scrollPosX = this.AutoScrollPosition.X;
            int scrollPosY = this.AutoScrollPosition.Y;
            graph.TranslateTransform(scrollPosX, scrollPosY);

            // on check si le document est ouvert :
            if (!documentHasData)
            {
                graph.DrawString("Document vide...", emptyFont, emptyBrush, new Point(20, 20));
                //FIXME
            }
            else
            {
                int minLine = WorldYCoordinateToLineIndex(e.ClipRectangle.Top - scrollPosY);
                if (minLine == -1)
                {
                    minLine = 0;
                }
                int maxLine = WorldYCoordinateToLineIndex(e.ClipRectangle.Bottom - scrollPosY);
                if ((maxLine >= this.documentLines.Count) ||
                    (maxLine == -1))
                {
                    maxLine = this.documentLines.Count-1;
                }
                TextLineInformation nextLine;
                for (int i = minLine;i<=maxLine;i++)
                {
                    nextLine = (TextLineInformation)documentLines[i];

                    graph.DrawString(nextLine.Text, mainFont, mainBrush, this.LineIndexToWorldCoordinates(i));
                }

            }
        }


        private Point LineIndexToWorldCoordinates(int i)
        {
            Point monPoint = new Point((int)margin, (int) (margin + i * lineHeight));
            return monPoint;
        }

        private int WorldYCoordinateToLineIndex(int y)
        {
            if (y < margin)
            {
                return -1;
            }
            else
            {
                return (int)((y - margin) / lineHeight);
            }
        } 
        
        private void CreateFonts()
        {
            mainFont = new Font("Arial", 10);
            lineHeight = (uint)mainFont.Height;
            emptyFont = new Font("Verdana", 13, FontStyle.Bold);
        }
        private void open_Click(object sender, EventArgs e)
        {
            fileOpenDialog.ShowDialog();
        }

        private void quitter_Click(object sender, EventArgs e)
        {
            this.Close();
        }


    }
}